@@ -1,4 +1,4 @@
-/*	$OpenBSD: crl.c,v 1.9 2020/09/12 15:46:48 claudio Exp $ */
+/*	$OpenBSD: crl.c,v 1.10 2021/01/29 10:13:16 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -29,14 +29,12 @@
 #include "extern.h"
 
 X509_CRL *
-crl_parse(const char *fn, const unsigned char *dgst)
+crl_parse(const char *fn)
 {
-	int		 rc = 0, sz;
+	int		 rc = 0;
 	X509_CRL	*x = NULL;
-	BIO		*bio = NULL, *shamd;
+	BIO		*bio = NULL;
 	FILE		*f;
-	EVP_MD		*md;
-	char		 mdbuf[EVP_MAX_MD_SIZE];
 
 	if ((f = fopen(fn, "rb")) == NULL) {
 		warn("%s", fn);
@@ -49,49 +47,11 @@ crl_parse(const char *fn, const unsigned char *dgst)
 		return NULL;
 	}
 
-	/*
-	 * If we have a digest specified, create an MD chain that will
-	 * automatically compute a digest during the X509 creation.
-	 */
-
-	if (dgst != NULL) {
-		if ((shamd = BIO_new(BIO_f_md())) == NULL)
-			cryptoerrx("BIO_new");
-		if (!BIO_set_md(shamd, EVP_sha256()))
-			cryptoerrx("BIO_set_md");
-		if ((bio = BIO_push(shamd, bio)) == NULL)
-			cryptoerrx("BIO_push");
-	}
-
 	if ((x = d2i_X509_CRL_bio(bio, NULL)) == NULL) {
 		cryptowarnx("%s: d2i_X509_CRL_bio", fn);
 		goto out;
 	}
 
-	/*
-	 * If we have a digest, find it in the chain (we'll already have
-	 * made it, so assert otherwise) and verify it.
-	 */
-
-	if (dgst != NULL) {
-		shamd = BIO_find_type(bio, BIO_TYPE_MD);
-		assert(shamd != NULL);
-
-		if (!BIO_get_md(shamd, &md))
-			cryptoerrx("BIO_get_md");
-		assert(EVP_MD_type(md) == NID_sha256);
-
-		if ((sz = BIO_gets(shamd, mdbuf, EVP_MAX_MD_SIZE)) < 0)
-			cryptoerrx("BIO_gets");
-		assert(sz == SHA256_DIGEST_LENGTH);
-
-		if (memcmp(mdbuf, dgst, SHA256_DIGEST_LENGTH)) {
-			if (verbose > 0)
-				warnx("%s: bad message digest", fn);
-			goto out;
-		}
-	}
-
 	rc = 1;
 out:
 	BIO_free_all(bio);
