@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.138 2021/04/15 14:22:05 claudio Exp $ */
+/*	$OpenBSD: main.c,v 1.141 2021/05/11 11:48:02 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -145,13 +145,14 @@ entityq_flush(struct entityq *q, struct repo *rp)
 	struct entity	*p, *np;
 
 	TAILQ_FOREACH_SAFE(p, q, entries, np) {
+		char *file = p->file;
+
 		/*
 		 * XXX fixup path here since the repo may change
 		 * during load because of fallback. In that case
 		 * the file path changes as well since RRDP and RSYNC
 		 * can not share a common repo.
 		 */
-		char *file = p->file;
 		p->file = repo_filename(rp, file);
 		if (p->file == NULL)
 			err(1, "can't construct repo filename");
@@ -405,6 +406,7 @@ queue_add_tal(const char *file)
 			err(1, NULL);
 	} else {
 		char *tmp;
+
 		if (asprintf(&tmp, "%s %s", stats.talnames, file) == -1)
 			err(1, NULL);
 		free(stats.talnames);
@@ -691,12 +693,6 @@ main(int argc, char *argv[])
 
 	signal(SIGPIPE, SIG_IGN);
 
-	if (timeout) {
-		signal(SIGALRM, suicide);
-		/* Commit suicide eventually - cron will normally start a new one */
-		alarm(timeout);
-	}
-
 	if (cachedir == NULL) {
 		warnx("cache directory required");
 		goto usage;
@@ -739,6 +735,9 @@ main(int argc, char *argv[])
 		if (fchdir(cachefd) == -1)
 			err(1, "fchdir");
 
+		if (timeout)
+			alarm(timeout);
+
 		/* Only allow access to the cache directory. */
 		if (unveil(".", "r") == -1)
 			err(1, "%s: unveil", cachedir);
@@ -773,6 +772,9 @@ main(int argc, char *argv[])
 			if (fchdir(cachefd) == -1)
 				err(1, "fchdir");
 
+			if (timeout)
+				alarm(timeout);
+
 			if (pledge("stdio rpath proc exec unveil", NULL) == -1)
 				err(1, "pledge");
 
@@ -809,6 +811,9 @@ main(int argc, char *argv[])
 			if (fchdir(cachefd) == -1)
 				err(1, "fchdir");
 
+			if (timeout)
+				alarm(timeout);
+
 			if (pledge("stdio rpath inet dns recvfd", NULL) == -1)
 				err(1, "pledge");
 
@@ -846,6 +851,9 @@ main(int argc, char *argv[])
 			if (fchdir(cachefd) == -1)
 				err(1, "fchdir");
 
+			if (timeout)
+				alarm(timeout);
+
 			if (pledge("stdio recvfd", NULL) == -1)
 				err(1, "pledge");
 
@@ -855,8 +863,19 @@ main(int argc, char *argv[])
 
 		close(fd[0]);
 		rrdp = fd[1];
-	} else
+	} else {
 		rrdp = -1;
+		rrdppid = -1;
+	}
+
+	if (timeout) {
+		/*
+		 * Commit suicide eventually
+		 * cron will normally start a new one
+		 */
+		alarm(timeout);
+		signal(SIGALRM, suicide);
+	}
 
 	/* TODO unveil cachedir and outputdir, no other access allowed */
 	if (pledge("stdio rpath wpath cpath fattr sendfd", NULL) == -1)
@@ -1027,6 +1046,7 @@ main(int argc, char *argv[])
 		}
 	}
 
+	signal(SIGALRM, SIG_DFL);
 	if (killme) {
 		syslog(LOG_CRIT|LOG_DAEMON,
 		    "excessive runtime (%d seconds), giving up", timeout);
