@@ -1,4 +1,4 @@
-/*      $OpenBSD: rrdp.c,v 1.5 2021/04/15 13:31:30 claudio Exp $ */
+/*	$OpenBSD: rrdp.c,v 1.9 2021/04/21 09:36:06 claudio Exp $ */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
@@ -272,6 +272,7 @@ rrdp_failed(struct rrdp *s)
 		s->sxml = new_snapshot_xml(s->parser, &s->current, s);
 		s->task = SNAPSHOT;
 		s->state = RRDP_STATE_REQ;
+		logx("%s: delta sync failed, fallback to snapshot", s->local);
 	} else {
 		/*
 		 * TODO: update state to track recurring failures
@@ -297,7 +298,6 @@ rrdp_finished(struct rrdp *s)
 		return;
 
 	if (s->state & RRDP_STATE_PARSE_ERROR) {
-		warnx("%s: failed after XML parse error", s->local);
 		rrdp_failed(s);
 		return;
 	}
@@ -331,22 +331,19 @@ rrdp_finished(struct rrdp *s)
 			s->last_mod = NULL;
 			switch (s->task) {
 			case NOTIFICATION:
-				warnx("%s: repository not modified",
-				    s->local);
+				logx("%s: repository not modified", s->local);
 				rrdp_state_send(s);
 				rrdp_free(s);
 				rrdp_done(id, 1);
 				break;
 			case SNAPSHOT:
-				warnx("%s: downloading snapshot",
-				    s->local);
+				logx("%s: downloading snapshot", s->local);
 				s->sxml = new_snapshot_xml(p, &s->current, s);
 				s->state = RRDP_STATE_REQ;
 				break;
 			case DELTA:
-				warnx("%s: downloading %lld deltas",
-				    s->local, s->repository.serial -
-				    s->current.serial);
+				logx("%s: downloading %lld deltas", s->local,
+				    s->repository.serial - s->current.serial);
 				s->dxml = new_delta_xml(p, &s->current, s);
 				s->state = RRDP_STATE_REQ;
 				break;
@@ -372,12 +369,11 @@ rrdp_finished(struct rrdp *s)
 			break;
 		}
 	} else if (s->res == HTTP_NOT_MOD && s->task == NOTIFICATION) {
-		warnx("%s: notification file not modified", s->local);
+		logx("%s: notification file not modified", s->local);
 		/* no need to update state file */
 		rrdp_free(s);
 		rrdp_done(id, 1);
 	} else {
-		warnx("%s: HTTP request failed", s->local);
 		rrdp_failed(s);
 	}
 }
